import {ActionType, PageContainer, ProForm, ProFormText,ProFormRadio,ProFormDigit, ProTable,} from '@ant-design/pro-components';
import {Button, Card, Modal, TreeSelect} from 'antd';
import React, {useEffect, useRef, useState} from 'react';
import {AddOutline} from 'antd-mobile-icons';
import {EditBundle, useEdit} from '@/core/hooks/useEdit';
import {invoke} from '@/core/network';

type MenuVO = {
  id?: number;
  name: string;
  sortNum: number;
  status: number;
  type: number;
  parentId?: number;
  url?: string;
  target?: string;
  hasRefresh?: number;
  perms?: string;
  icon?: string;
  children?: MenuVO[];
};

type MenuList = MenuVO[];

class Service {

  public static async listTree(query: { keyword?: string; status?: number }) {
    return invoke<MenuList>('/apis/admin/menu/tree/list', {
      method: 'POST',
      data: query,
    });
  }

  public static async remove(id: number) {
    return invoke<void>(`/apis/admin/menu/remove?id=${id}`, {
      method: 'DELETE',
    });
  }

  public static async save(menu: Partial<MenuVO>) {
    return invoke<MenuVO>('/apis/admin/menu/save', {
      method: 'POST',
      data: menu,
    });
  }

  public static async get(id: number) {
    return invoke<MenuVO>(`/apis/admin/menu/details?id=${id}`, {
      method: 'GET',
    });
  }
}

const Edit: React.FC<{ bundle: EditBundle; treeData: any[]; onSuccess?: () => void }> = (props) => {
  const {bundle, treeData} = props;
  console.log("treeData" + treeData)

  const [form] = ProForm.useForm<MenuVO>();
  const [loading, setLoading] = React.useState(false);

  const [value, setValue] = useState<string>();

  const onChange = (newValue: string) => {
    console.log(newValue);
    setValue(newValue);
  };

  // @ts-ignore
  // @ts-ignore
  // @ts-ignore
  return (
    <Modal
      confirmLoading={loading}
      title={bundle.id ? '编辑菜单' : '添加菜单'}
      open={bundle.open}
      onOk={async () => {
        await form.validateFields();

        try {
          setLoading(true);

          await Service.save({
            id: bundle.id,
            ...form.getFieldsValue(),
          });
        } finally {
          setLoading(false);
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onClose={() => {
        form.resetFields();
        bundle.close();
      }}
      onCancel={() => {
        form.resetFields();
        bundle.close();
      }}
    >
      <ProForm
        className={'mt-5'}
        form={form}
        labelCol={{span: 4}}
        layout={'horizontal'}
        submitter={false}
        request={async () => {
          if (bundle.id !== undefined) {
            console.log(bundle.id, 'bundle');
            const res = await Service.get(bundle.id);

            return res.data;
          }

          return Promise.resolve({});
        }}
      >

        <ProFormRadio.Group name="type" label="类型" value='2' options={[{label: '目录', value: '1',}, {label: '菜单', value: '2'}, {label: '按钮', value: '3'}]}/>
        <ProFormRadio.Group name="status" label="状态" value='1' options={[{label: '正常', value: '1',}, {label: '隐藏', value: '2'}]}/>
        <ProFormRadio.Group name="target" label="目标" value='1' options={[{label: '页签', value: '1',}, {label: '新窗口', value: '2'}]}/>
        <ProFormRadio.Group name="hasRefresh" label="是否刷新" value='1' options={[{label: '不刷新', value: '0',}, {label: '刷新', value: '1'}]}/>
        <ProFormText name='name' label={'菜单名称'} placeholder={'请输入菜单名称'}
                     rules={[{required: true, message: '请输入菜单名称',}, {
                       min: 3,
                       max: 30,
                       message: '菜单名称长度必须在3-30之间',
                     },]}></ProFormText>
        <ProForm.Item label='父菜单' name='parentId'>
          <TreeSelect
            showSearch
            style={{width: '100%'}}
            value={value}
            dropdownStyle={{maxHeight: 400, overflow: 'auto'}}
            placeholder="请选择父菜单ID"
            allowClear
            treeDefaultExpandAll
            onChange={onChange}
            treeData={treeData}
          />
        </ProForm.Item>
        <ProFormText name='icon' label={'图标'} placeholder={'请输入图标'}></ProFormText>

        <ProFormDigit name='sortNum' label={'显示顺序'} placeholder={'请输入显示顺序'}></ProFormDigit>
        <ProFormText name='url' label={'请求地址'} placeholder={'请输入请求地址'}></ProFormText>
        <ProFormText name='perms' label={'权限标识'} placeholder={'请输入权限标识'}></ProFormText>
      </ProForm>
    </Modal>
  );
};

const transformToTreeData = (data: MenuVO[]): any[] =>
  data.map((item) => ({
    title: item.name,
    value: item.id,
    key: item.id,
    children: item.children ? transformToTreeData(item.children) : [],
  }));
const MenuExp: React.FC = () => {
  const bundle = useEdit();
  const ref = useRef<ActionType>();

  const [treeData, setTreeData] = useState<any[]>([]);

  const loadData = async () => {
    const res = await Service.listTree({});
    if (res.success) {
      setTreeData(transformToTreeData(res.data));
    }
  };

  useEffect(() => {
    loadData();
  }, []);

  return (
    <>
      <PageContainer content={''}>
        {bundle.open && <Edit bundle={bundle} treeData={treeData} onSuccess={() => ref.current?.reload()}></Edit>}
        <Card>
          <ProTable<MenuVO>
            rowKey={'id'}
            request={async (params) => {
              const res = await Service.listTree({});

              return {
                success: res.success,
                data: res.data,
              };
            }}

            columns={[
              {
                title: 'ID',
                dataIndex: 'id',
              },
              {
                title: '菜单名称',
                dataIndex: 'name',
              },
              {
                title: '类型',
                dataIndex: 'type',
              },
              {
                title: '父菜单ID',
                dataIndex: 'parentId',
              },
              {
                title: '显示顺序',
                dataIndex: 'sortNum',
              },
              {
                title: '请求地址',
                dataIndex: 'url',
              },
              {
                title: '目标',
                dataIndex: 'target',
              },
              {
                title: '状态',
                dataIndex: 'status',
              },
              {
                title: '是否刷新',
                dataIndex: 'hasRefresh',
              },
              {
                title: '权限标识',
                dataIndex: 'perms',
              },
              {
                title: '操作',
                valueType: 'option',
                render: (_, record) => [
                  <a key={'edit'} onClick={() => bundle.update(record.id as number)}>
                    编辑
                  </a>,
                  <a
                    key={'delete'}
                    onClick={async () => {
                      await Service.remove(record.id as number);
                      ref.current?.reload();
                    }}
                  >
                    删除
                  </a>,
                ],
              },
            ]}

            actionRef={ref}
            toolbar={{
              actions: [
                <Button icon={<AddOutline/>} key={'add'} type={'primary'} onClick={bundle.create}>
                  添加
                </Button>,
              ],
              settings: [],
            }}
            search={false}
          ></ProTable>
        </Card>
      </PageContainer>
    </>
  );
};

export default MenuExp;
