import {
  ActionType,
  PageContainer,
  ProForm,
  ProFormText,
  ProTable,
} from '@ant-design/pro-components';
import { Button, Card, Modal } from 'antd';
import React, { useRef } from 'react';
import { AddOutline } from 'antd-mobile-icons';
import { EditBundle, useEdit } from '@/core/hooks/useEdit';
import { invoke } from '@/core/network';

type Role = {
  id?: number;
  name: string;
  sortNum: number;
  status: number;
};

type RoleList = {
  records: Role[];
  total: number;
  size: number;
  current: number;
};

class Service {
  public static async list(current: number, size: number) {
    return invoke<RoleList>('/apis/admin/role/page', {
      method: 'POST',
      data: {
        current,
        size,
      },
    });
  }

  public static async remove(id: number) {
    return invoke<void>(`/apis/admin/role/remove?id=${id}`, {
      method: 'DELETE',
    });
  }

  public static async save(member: Partial<Role>) {
    return invoke<Role>('/apis/admin/role/save', {
      method: 'POST',
      data: member,
    });
  }

  public static async get(id: number) {
    return invoke<Role>(`/apis/admin/role/details?id=${id}`, {
      method: 'GET',
    });
  }
}

const Edit: React.FC<{ bundle: EditBundle; onSuccess?: () => void }> = (props) => {
  const { bundle } = props;
  const [form] = ProForm.useForm<Role>();
  const [loading, setLoading] = React.useState(false);

  return (
    <Modal
      confirmLoading={loading}
      title={bundle.id ? '编辑角色' : '添加角色'}
      open={bundle.open}
      onOk={async () => {
        await form.validateFields();

        try {
          setLoading(true);

          await Service.save({
            id: bundle.id,
            ...form.getFieldsValue(),
          });
        } finally {
          setLoading(false);
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onClose={() => {
        form.resetFields();
        bundle.close();
      }}
      onCancel={() => {
        form.resetFields();
        bundle.close();
      }}
    >
      <ProForm
        className={'mt-5'}
        form={form}
        labelCol={{ span: 4 }}
        layout={'horizontal'}
        submitter={false}
        request={async () => {
          if (bundle.id !== undefined) {
            console.log(bundle.id, 'bundle');
            const res = await Service.get(bundle.id);

            return res.data;
          }

          return Promise.resolve({});
        }}
      >
        <ProFormText
          name={'name'}
          label={'角色名称'}
          rules={[
            {
              required: true,
              message: '请输入角色名称',
            },
            {
              min: 3,
              max: 30,
              message: '角色名称长度必须在3-30之间',
            },
          ]}
          placeholder={'请输入角色名称'}
        ></ProFormText>
      </ProForm>
    </Modal>
  );
};

const RoleExp: React.FC = () => {
  const bundle = useEdit();
  const ref = useRef<ActionType>();

  return (
    <PageContainer content={''}>
      {bundle.open && <Edit bundle={bundle} onSuccess={() => ref.current?.reload()}></Edit>}
      <Card>
        <ProTable<Role>
          rowKey={'id'}
          request={async (params) => {
            const res = await Service.list(params.current || 1, params.pageSize || 10);

            return {
              success: res.success,
              total: res.data.total,
              data: res.data.records,
            };
          }}
          columns={[
            {
              title: 'ID',
              dataIndex: 'id',
            },
            {
              title: '角色名称',
              dataIndex: 'name',
            },
            {
              title: '操作',
              valueType: 'option',
              render: (_, record) => [
                <a key={'edit'} onClick={() => bundle.update(record.id as number)}>
                  编辑
                </a>,
                <a
                  key={'delete'}
                  onClick={async () => {
                    await Service.remove(record.id as number);
                    ref.current?.reload();
                  }}
                >
                  删除
                </a>,
              ],
            },
          ]}
          actionRef={ref}
          toolbar={{
            actions: [
              <Button icon={<AddOutline />} key={'add'} type={'primary'} onClick={bundle.create}>
                添加
              </Button>,
            ],
            settings: [],
          }}
          search={false}
        ></ProTable>
      </Card>
    </PageContainer>
  );
};

export default RoleExp;
