import {
  ActionType,
  PageContainer,
  ProForm,
  ProFormText,
  ProTable,
} from '@ant-design/pro-components';
import { Button, Card, Modal } from 'antd';
import React, { useRef } from 'react';
import { AddOutline } from 'antd-mobile-icons';
import { EditBundle, useEdit } from '@/core/hooks/useEdit';
import { invoke } from '@/core/network';

type Member = {
  id?: number;
  username: string;
  password: string;
  nickname: string;
  thumbAvatar?: string;
  level: number;
};

type MemberList = {
  records: Member[];
  total: number;
  size: number;
  current: number;
};

class Service {
  public static async list(current: number, size: number) {
    return invoke<MemberList>('/apis/admin/staff/page', {
      method: 'POST',
      data: {
        current,
        size,
      },
    });
  }

  public static async remove(id: number) {
    return invoke<void>(`/apis/admin/staff/remove?id=${id}`, {
      method: 'DELETE',
    });
  }

  public static async save(member: Partial<Member>) {
    return invoke<Member>('/apis/admin/staff/save', {
      method: 'POST',
      data: member,
    });
  }

  public static async get(id: number) {
    return invoke<Member>(`/apis/admin/staff/details?id=${id}`, {
      method: 'GET',
    });
  }
}

const Edit: React.FC<{ bundle: EditBundle; onSuccess?: () => void }> = (props) => {
  const { bundle } = props;
  const [form] = ProForm.useForm<Member>();
  const [loading, setLoading] = React.useState(false);

  return (
    <Modal
      confirmLoading={loading}
      title={bundle.id ? '编辑管理员' : '添加管理员'}
      open={bundle.open}
      onOk={async () => {
        await form.validateFields();

        try {
          setLoading(true);

          await Service.save({
            id: bundle.id,
            ...form.getFieldsValue(),
            thumbAvatar: '',
            level: 1,
          });
        } finally {
          setLoading(false);
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onClose={() => {
        form.resetFields();
        bundle.close();
      }}
      onCancel={() => {
        form.resetFields();
        bundle.close();
      }}
    >
      <ProForm
        className={'mt-5'}
        form={form}
        labelCol={{ span: 4 }}
        layout={'horizontal'}
        submitter={false}
        request={async () => {
          if (bundle.id !== undefined) {
            console.log(bundle.id, 'bundle');
            const res = await Service.get(bundle.id);

            return res.data;
          }

          return Promise.resolve({});
        }}
      >
        <ProFormText
          name={'username'}
          label={'账号'}
          rules={[
            {
              required: true,
              message: '请输入账号',
            },
            {
              min: 3,
              max: 20,
              message: '账号长度必须在3-20之间',
            },
          ]}
          placeholder={'请输入账号'}
        ></ProFormText>
        <ProFormText
          name="nickname"
          label="姓名"
          rules={[
            {
              required: true,
              message: '请输入姓名',
            },
            {
              min: 2,
              max: 20,
              message: '姓名长度必须在2-20之间',
            },
          ]}
          placeholder={'请输入用户姓名'}
        />
        <ProFormText.Password
          name="password"
          label="密码"
          placeholder={'请输入密码'}
          rules={[
            {
              required: true,
              message: '请输入密码',
            },
            {
              min: 6,
              max: 50,
              message: '密码长度必须在6-50之间',
            },
          ]}
        />
      </ProForm>
    </Modal>
  );
};

const Staff: React.FC = () => {
  const bundle = useEdit();
  const ref = useRef<ActionType>();

  return (
    <PageContainer content={''}>
      {bundle.open && <Edit bundle={bundle} onSuccess={() => ref.current?.reload()}></Edit>}
      <Card>
        <ProTable<Member>
          rowKey={'id'}
          request={async (params) => {
            const res = await Service.list(params.current || 1, params.pageSize || 10);

            return {
              success: res.success,
              total: res.data.total,
              data: res.data.records,
            };
          }}
          columns={[
            {
              title: 'ID',
              dataIndex: 'id',
            },
            {
              title: '账号',
              dataIndex: 'username',
            },
            {
              title: '姓名',
              dataIndex: 'nickname',
            },
            {
              title: '操作',
              valueType: 'option',
              render: (_, record) => [
                <a key={'edit'} onClick={() => bundle.update(record.id as number)}>
                  编辑
                </a>,
                <a
                  key={'delete'}
                  onClick={async () => {
                    await Service.remove(record.id as number);
                    ref.current?.reload();
                  }}
                >
                  删除
                </a>,
              ],
            },
          ]}
          actionRef={ref}
          toolbar={{
            actions: [
              <Button icon={<AddOutline />} key={'add'} type={'primary'} onClick={bundle.create}>
                添加
              </Button>,
            ],
            settings: [],
          }}
          search={false}
        ></ProTable>
      </Card>
    </PageContainer>
  );
};

export default Staff;
